﻿//----------------------------------------------------------------------------
// <copyright file="Pluralizer.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public static class Pluralizer
    {
        private static char[] Capitals = new char[] { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' };

        private static Dictionary<string, string> plurals = new Dictionary<string, string>()
        {
            {"Account","Accounts"},
            {"Action","Actions"},
            {"Address","Addresses"},
            {"Archive","Archives"},
            {"Asset","Assets"},
            {"Author","Authors"},
            {"Band","Bands"},
            {"Batch","Batches"},
            {"Book","Books"},
            {"Brand","Brands"},
            {"Candidates","Candidates"},
            {"Card","Cards"},
            {"Category","Categories"},
            {"Class","Classes"},
            {"Code","Codes"},
            {"Color","Colours"},
            {"Condition","Conditions"},
            {"Contact","Contacts"},
            {"Control","Controls"},
            {"Conversation","Conversations"},
            {"Converter","Converters"},
            {"Cost","Costs"},
            {"Country","Countries"},
            {"Culture","Cultures"},
            {"Currency","Currencies"},
            {"Customer","Customers"},
            {"Date","Dates"},
            {"Day","Days"},
            {"Delivery","Deliveries"},
            {"Department","Departments"},
            {"Description","Descriptions"},
            {"Detail","Details"},
            {"Discount","Discounts"},
            {"Document","Documents"},
            {"Download","Downloads"},
            {"Driver","Drivers"},
            {"Edition","Editions"},
            {"Element","Elements"},            
            {"Employee","Employees"},            
            {"Fee","Fees"},
            {"Family","Families"},
            {"File","Files"},
            {"Form","Forms"},
            {"Function","Functions"},
            {"Global","Globals"},
            {"Group","Groups"},
            {"Illustration","Illustrations"},
            {"Individual","Individuals"},
            {"Inventory","Inventories"},
            {"Invoice","Invoices"},
            {"Item","Items"},
            {"Job","Jobs"},
            {"Journal","Journals"},
            {"Ledger","Ledgers"},
            {"Level","Levels"},
            {"Line","Lines"},
            {"List","Lists"},
            {"Load","Loads"},
            {"Location","Locations"},
            {"Manager","Managers"},
            {"Map","Maps"},
            {"Material","Materials"},
            {"Measure","Measures"},
            {"Member","Members"},
            {"Message","Messages"},
            {"Method","Methods"},
            {"Metric","Metrics"},
            {"Model","Models"},
            {"Movement","Movements"},
            {"Note","Notes"},
            {"On","Ons"},
            {"Offer","Offers"},
            {"Option","Options"},
            {"Order","Orders"},
            {"Password","Passwords"},
            {"Person","People"},
            {"Period","Periods"},
            {"Photo","Photos"},
            {"Policy","Policies"},
            {"Price","Prices"},
            {"Processor","Processors"},
            {"Product","Products"},
            {"Profile","Profiles"},
            {"Proforma","Proformas"},
            {"Province","Provinces"},
            {"Publisher","Publishers"},
            {"Quote","Quotes"},
            {"Rate","Rates"},
            {"Reason","Reasons"},
            {"Rebate","Rebates"},
            {"Record","Records"},
            {"Reference","References"},
            {"Region","Regions"},
            {"Request","Requests"},
            {"Review","Reviews"},
            {"Role","Roles"},
            {"Room","Rooms"},
            {"Routing","Routings"},
            {"Rule","Rules"},
            {"Schedule","Schedules"},
            {"Session","Sessions"},
            {"Shift","Shifts"},
            {"Shipment","Shipments"},
            {"Size","Sizes"},
            {"Statistic","Statistics"},
            {"Status","Statuses"},
            {"Store","Stores"},
            {"Stream","Streams"},
            {"Style","Styles"},
            {"Subcategory","Subcategory"},
            {"Supplier","Suppliers"},
            {"System","Systems"},
            {"Task","Tasks"},
            {"Tax","Taxes"},
            {"Team","Teams"},
            {"Term","Terms"},
            {"Territory","Territories"},
            {"Title","Titles"},
            {"Topic","Topics"},
            {"Type","Types"},
            {"Transaction","Transactions"},
            {"User","Users"},
            {"Underwriter","Underwriters"},
            {"Variable","Variables"},
            {"Variation","Variations"},
            {"Vendor","Vendors"},
            {"Warehouse","Warehouses"}

        };

        public static PluralType Mode
        {
            get;
            set;
        } 

        public static string Singlular(this string text)
        {
            if (Mode == PluralType.None)
            {
                return text;
            }

            string plural = text.EndWord();

            var single = (from p in plurals
                          where p.Value == plural
                          select p.Key).FirstOrDefault();

            if (!string.IsNullOrEmpty(single))
            {
                return text.TrimEnd(plural.Length) + single;
            }
            else
            {
                return text;
            }
        }

        public static string EndWord(this string text)
        {
            int i = text.LastIndexOfAny(Capitals);
            if (i >= 0)
            {
                return text.Substring(i);
            }
            else
            {
                return text;
            }
        }

        public static string Plural(this string text)
        {
            if (Mode == PluralType.None)
            {
                return text;
            }

            string single = text.EndWord();

            if (plurals.ContainsKey(single))
            {
                return text.TrimEnd(single.Length) + plurals[single];
            }
            else
            {
                return text;
            }
        }
    }
}
