﻿//-----------------------------------------------------------------------
// <copyright file="DataAccessInterfaceProfile.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax
{
    using System.ComponentModel;
    using System.IO;

    /// <summary>
    /// Class to hold the properties used for the Data Access Interface generation.
    /// </summary>
    public class DataAccessInterfaceProfile : INotifyPropertyChanged
    {
        private bool isDirty = false;
        private string generationNamespace;
        private string outputPath;
        private string projectFile;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DataAccessInterfaceProfile class.
        /// </summary>
        public DataAccessInterfaceProfile()
        {
        }
        #endregion

        /// <summary>
        /// INotifyPropertyChanged event.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region Public properties
        /// <summary>
        /// Gets or sets the Namespace.
        /// </summary>
        /// <value>The Namespace.</value>
        public string Namespace
        {
            get
            {
                return this.generationNamespace;
            }

            set
            {
                if (this.generationNamespace != value)
                {
                    this.generationNamespace = value;
                    this.OnPropertyChanged("Namespace");
                }
            }
        }

        /// <summary>
        /// Gets or sets the OutputPath.
        /// </summary>
        /// <value>The OutputPath.</value>
        public string OutputPath
        {
            get
            {
                return this.outputPath;
            }

            set
            {
                if (this.outputPath != value)
                {
                    this.outputPath = value;
                    this.OnPropertyChanged("OutputPath");
                }
            }
        }

        /// <summary>
        /// Gets or sets the Visual Studio project file.
        /// </summary>
        public string VSProjectFile
        {
            get
            {
                return this.projectFile;
            }

            set
            {
                if (this.projectFile != value)
                {
                    this.projectFile = value;
                    this.OnPropertyChanged("VSProjectFile");
                }
            }
        }
        #endregion

        internal bool IsDirty
        {
            get
            {
                return this.isDirty;
            }

            set
            {
                this.isDirty = value;
            }
        }


        /// <summary>
        /// Gets the full path for a generated file in the base output path.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string OutputFilePath(string fileName)
        {
            return Path.Combine(this.OutputPath, fileName);
        }

        private void OnPropertyChanged(string name)
        {
            this.isDirty = true;

            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }
    }
}
