﻿//-----------------------------------------------------------------------
// <copyright file="StoredProcedureProfile.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax
{
    using System.ComponentModel;

    /// <summary>
    /// Class to hold the properties used for stored procedure generation.
    /// </summary>
    public class StoredProcedureProfile : INotifyPropertyChanged
    {
        private bool isDirty = false;
        private string outputFile;
        private string prefix;
        private string userPrefix;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the StoredProcedureProfile class.
        /// </summary>
        public StoredProcedureProfile()
        {
        }
        #endregion

        /// <summary>
        /// INotifyPropertyChanged event.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region Public Properties
        /// <summary>
        /// Gets or sets the output file for the stored procedure sql script.
        /// </summary>
        public string OutputFile
        {
            get
            {
                return this.outputFile;
            }

            set
            {
                if (this.outputFile != value)
                {
                    this.outputFile = value;
                    this.OnPropertyChanged("OutputFile");
                }
            }
        }

        /// <summary>
        /// Gets or sets the prefix used to identify generated stored procedures.
        /// </summary>
        public string Prefix
        {
            get
            {
                return this.prefix;
            }

            set
            {
                if (this.prefix != value)
                {
                    this.prefix = value;
                    this.OnPropertyChanged("Prefix");
                }
            }
        }

        /// <summary>
        /// Gets or sets the prefix used to identify user created stored procedures.
        /// </summary>
        public string UserPrefix
        {
            get
            {
                return this.userPrefix;
            }

            set
            {
                if (this.userPrefix != value)
                {
                    this.userPrefix = value;
                    this.OnPropertyChanged("UserPrefix");
                }
            }
        }
        #endregion

        internal bool IsDirty
        {
            get
            {
                return this.isDirty;
            }

            set
            {
                this.isDirty = value;
            }
        }

        private void OnPropertyChanged(string name)
        {
            this.isDirty = true;

            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }
    }
}
