﻿//---------------------------------------------------------------------------------
// <copyright file="VSProject.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//---------------------------------------------------------------------------------
namespace Dax
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Microsoft.Build.Evaluation;
    using System.IO;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class VSProject
    {
        private Project project;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the VSProject class.
        /// </summary>
        public VSProject(string fileName)
        {
            this.project = new Project(fileName);
        }
        #endregion

        public void ClearFolder(string folder)
        {
            string d = folder + @"\";

            // Remove files not in the folder.
            List<ProjectItem> removeItems = new List<ProjectItem>();
            foreach (var item in this.project.GetItems("compile"))
            {
                if (item.EvaluatedInclude.StartsWith(d, StringComparison.OrdinalIgnoreCase))
                {
                    string fileFullPath = Path.Combine(this.project.DirectoryPath, item.EvaluatedInclude);
                    if (!File.Exists(fileFullPath))
                    {
                        removeItems.Add(item);
                    }
                }
            }
            this.project.RemoveItems(removeItems);
        }

        public void AddCompileItem(string fileName)
        {
            string item;

            if (Path.GetDirectoryName(fileName) == this.project.DirectoryPath)
            {
                item = Path.GetFileName(fileName);
            }
            else
            {
                string folder = Path.GetDirectoryName(fileName).Substring(this.project.DirectoryPath.Length + 1);
                item = Path.Combine(folder, Path.GetFileName(fileName));
            }

            var pi = this.project.GetItemsByEvaluatedInclude(item);

            if (pi.Count == 0)
            {
                this.project.AddItem("compile", item);
            }
        }

        public void AddDesignerCompileItem(string fileName)
        {
            string p = Path.GetDirectoryName(fileName);
            string folder = Path.GetFileNameWithoutExtension(p);

            // Find the parent.
            string parentFileName = Path.GetFileNameWithoutExtension(Path.GetFileNameWithoutExtension(fileName)) + Path.GetExtension(fileName);
            Dictionary<string,string> mdata = new Dictionary<string,string>();
            mdata.Add("DependentUpon", parentFileName);

            string item = Path.Combine(folder, Path.GetFileName(fileName));
            var pi = this.project.GetItemsByEvaluatedInclude(item);

            if (pi.Count == 0)
            {
                this.project.AddItem("compile", item, mdata);
            }
        }

        public void Save()
        {
            if (this.project.IsDirty)
            {
                this.project.Save();
            }
        }

    }
}
