﻿//-----------------------------------------------------------------------
// <copyright file="DBColumn.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All Rights Reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Database
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DBColumn : DBObject
    {
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DBColumn class.
        /// </summary>
        /// <param name="dbDatabase">The DBDatabase this column belongs to.</param>
        /// <param name="id">The column id.</param>
        /// <param name="name">The name of the column.</param>
        public DBColumn(DBDatabase dbDatabase, int id, string name)
            : base(dbDatabase, id, name)
        {
        }
        #endregion

        /// <summary>
        /// Gets or sets a value indicating whether the column is nullable.
        /// </summary>
        public bool IsNullable
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the column is a computed value.
        /// </summary>
        public bool IsComputed
        {
            get;
            set;
        }

        /// <summary>
        ///  Gets or sets a value indicating whether this column is an identity column.
        /// </summary>
        public bool IsIdentity
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the
        /// </summary>
        public int MaxLength
        {
            get;
            set;
        }

        /// <summary>
        ///  Gets or sets the precision.
        /// </summary>
        public int Precision
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the scale.
        /// </summary>
        public int Scale
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the column is part of the primary key.
        /// </summary>
        public bool IsPrimaryKey
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the column is part of a foreign key.
        /// </summary>
        public bool IsForeignKey
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the UserType.
        /// </summary>
        public DBType UserType
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the SystemType.
        /// </summary>
        public DBType SystemType
        {
            get;
            set;
        }

        /// <summary>
        /// Gets a string containing the type declaration of the column.
        /// </summary>
        public string SqlType
        {
            get
            {
                string t = this.SystemType.Name;

                if (this.SystemType.Name == "char" || this.SystemType.Name == "varchar"
                    || this.SystemType.Name == "nchar" || this.SystemType.Name == "nvarchar")
                {
                    t += "(";
                    if (this.MaxLength == -1)
                    {
                        t += "max";
                    }
                    else
                    {
                        if (this.SystemType.Name.StartsWith("n", StringComparison.OrdinalIgnoreCase))
                        {
                            t += this.MaxLength / 2;
                        }
                        else
                        {
                            t += this.MaxLength;
                        }
                    }

                    t += ")";
                }

                return t;
            }
        }
    }
}
