﻿//-----------------------------------------------------------------------
// <copyright file="DBDatabase.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Database
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DBDatabase
    {
        private Dictionary<int, DBSchema> schemas = new Dictionary<int, DBSchema>();
        private Dictionary<int, DBType> types = new Dictionary<int, DBType>();
        private Dictionary<int, DBTable> tables = new Dictionary<int, DBTable>();
        private Dictionary<int, DBRelationship> relationships = new Dictionary<int, DBRelationship>();
        private Dictionary<int, DBProcedure> procedures = new Dictionary<int, DBProcedure>();

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DatabaseInfo class.
        /// </summary>
        public DBDatabase(string name)
        {
            this.Name = name;
        }
        #endregion

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the Database Types.
        /// </summary>
        /// <value>The Database Ttpes.</value>
        public Dictionary<int, DBType> Types
        {
            get
            {
                return this.types;
            }
        }

        /// <summary>
        /// Gets the Database Schemas.
        /// </summary>
        /// <value>The Database Ttpes.</value>
        public Dictionary<int, DBSchema> Schemas
        {
            get
            {
                return this.schemas;
            }
        }

        /// <summary>
        /// Gets the Database Schemas.
        /// </summary>
        /// <value>The Database Ttpes.</value>
        public Dictionary<int, DBTable> Tables
        {
            get
            {
                return this.tables;
            }
        }

        /// <summary>
        /// Gets the Database Schemas.
        /// </summary>
        /// <value>The Database Ttpes.</value>
        public Dictionary<int, DBProcedure> Procedures
        {
            get
            {
                return this.procedures;
            }
        }

        /// <summary>
        /// Gets the Database Schemas.
        /// </summary>
        /// <value>The Database Ttpes.</value>
        public Dictionary<int, DBRelationship> Relationships
        {
            get
            {
                return this.relationships;
            }
        }

        public void RemoveTable(string name)
        {
            Trace.TraceInformation("Removing table " + name);
            DBTable table = (from t in this.tables.Values
                             where t.Name == name
                             select t).First();

            if (this.tables == null)
            {
                return;
            }

            // Remove any relationships to this table.
            List<int> deletes = new List<int>();
            foreach (var item in this.Relationships.Values.Where(r => r.ParentTable == table || r.ReferencedTable == table))
            {
                Trace.TraceInformation("Removing relationship " + item.Name);
                deletes.Add(item.Id);
            }

            foreach (int objectId in deletes)
            {
                this.relationships.Remove(objectId);
            }

            this.tables.Remove(table.Id);
        }

        public void RemoveRelationship(DBRelationship relationship)
        {
        }

    }
}
