﻿//----------------------------------------------------------------------------
// <copyright file="DbObject.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax.Database
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Base class for all database objects.
    /// </summary>
    public abstract class DBObject
    {
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DbObject class.
        /// </summary>
        /// <param name="dbDatabase">The DBDatabase the object belongs to.</param>
        /// <param name="id">The id of the object.</param>
        /// <param name="name">The name of the object.</param>
        protected DBObject(DBDatabase dbDatabase, int id, string name)
        {
            this.Name = name;
            this.Id = id;
            this.DBDatabase = dbDatabase;
        }
        #endregion

        /// <summary>
        /// Gets the name.
        /// </summary>
        public string Name
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the id.
        /// </summary>
        public int Id
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the Database this object belongs to.
        /// </summary>
        public DBDatabase DBDatabase
        {
            get;
            private set;
        }
    }
}
