﻿//-----------------------------------------------------------------------
// <copyright file="DBTable.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Database
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DBTable : DBObject
    {
        private Dictionary<int, DBColumn> columns = new Dictionary<int, DBColumn>();
        private Dictionary<int, DBColumn> primaryKeys = new Dictionary<int, DBColumn>();

        /// <summary>
        /// Gets the summary.
        /// </summary>
        /// <value>The summary.</value>
        public DBSchema Schema
        {
            get;
            private set;
        }

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DBTable class.
        /// </summary>
        public DBTable(DBDatabase dbDatabase, DBSchema schema, int objectId, string name) 
            :base(dbDatabase, objectId,name)
        {
            this.Schema = schema;
        }
        #endregion

        /// <summary>
        /// Gets the columns.
        /// </summary>
        /// <value>The Database Types.</value>
        public Dictionary<int, DBColumn> Columns
        {
            get
            {
                return this.columns;
            }
        }

        public bool HasCompositePrimaryKey
        {
            get
            {
                return this.PrimaryKey.Count > 1;
            }
        }

        public bool HasIdentity
        {
            get
            {
                return (this.Identity != null);
            }
        }

        public DBColumn Identity
        {
            get
            {
                return (from c in this.Columns.Values
                        where c.IsIdentity
                        select c).FirstOrDefault();
            }
        }

        /// <summary>
        /// Gets the columns.
        /// </summary>
        /// <value>The Database Types.</value>
        public Dictionary<int, DBColumn> PrimaryKey
        {
            get
            {
                return this.primaryKeys;
            }
        }

        public void RemoveColumn(DBColumn column)
        {
            this.columns.Remove(column.Id);
        }

        public IEnumerable<DBRelationship> ParentRelationships
        {
            get
            {
                return this.DBDatabase.Relationships.Values.Where(x => x.ParentTable == this).ToList();
            }
        }

        public IEnumerable<DBRelationship> ReferencedRelationships
        {
            get
            {
                return this.DBDatabase.Relationships.Values.Where(x => x.ReferencedTable == this).ToList();
            }
        }
    }
}
