﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Dax;

namespace Editor
{
    public partial class ProfileControl : UserControl
    {
        public GenerationProfile profile;
        BindingList<string> excludesBindingList;

        public ProfileControl()
        {
            SetStyle(ControlStyles.OptimizedDoubleBuffer |
                     ControlStyles.AllPaintingInWmPaint, true);

            InitializeComponent();
        }

        public void SetProfile(GenerationProfile profile)
        {
            this.profile = profile;

            this.BeginUpdate(true);
            this.connectionStringTextBox.Rebind("Text", this.profile, "ConnectionString", false, DataSourceUpdateMode.OnPropertyChanged);
            this.traceCheckBox.Rebind("Checked", this.profile, "Trace", false, DataSourceUpdateMode.OnPropertyChanged);

            this.excludesBindingList = new BindingList<string>(this.profile.TableExclusions);
            this.excudesListBox.DataSource = this.excludesBindingList;

            int p = (int)this.profile.Pluralizer; ;
            this.pluralizerComboBox.DataSource = System.Enum.GetValues(typeof(PluralType));
            this.pluralizerComboBox.SelectedIndex = p;

            this.sprocOutputFileTextBox.Rebind("Text", this.profile.StoredProcedures, "OutputFile", false, DataSourceUpdateMode.OnPropertyChanged);
            this.sprocPrefixTextBox.Rebind("Text", this.profile.StoredProcedures, "Prefix", false, DataSourceUpdateMode.OnPropertyChanged);
            this.sprocUserPrefixTextBox.Rebind("Text", this.profile.StoredProcedures, "UserPrefix", false, DataSourceUpdateMode.OnPropertyChanged);

            this.daBuildersFolderTextBox.Rebind("Text", this.profile.DataAccess, "BuildersFolder", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daNamespaceTextBox.Rebind("Text", this.profile.DataAccess, "Namespace", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daOutputPathTextBox.Rebind("Text", this.profile.DataAccess, "OutputPath", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daProjectFileTextBox.Rebind("Text", this.profile.DataAccess, "VSProjectFile", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daTestProjectFileTextBox.Rebind("Text", this.profile.DataAccess, "TestVSProjectFile", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daRecordsFolderTextBox.Rebind("Text", this.profile.DataAccess, "RecordsFolder", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daRepositoriesFolderTextBox.Rebind("Text", this.profile.DataAccess, "RepositoriesFolder", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daConcurrentColumnTextBox.Rebind("Text", this.profile.DataAccess, "ConcurrencyColumn", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daTestNamespaceTextBox.Rebind("Text", this.profile.DataAccess, "TestNamespace", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daTestOutputPathTextBox.Rebind("Text", this.profile.DataAccess, "TestOutputPath", false, DataSourceUpdateMode.OnPropertyChanged);

            this.daiProjectFileTextBox.Rebind("Text", this.profile.DataAccessInterface, "VSProjectFile", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daiNamespaceTextBox.Rebind("Text", this.profile.DataAccessInterface, "Namespace", false, DataSourceUpdateMode.OnPropertyChanged);
            this.daiOutputPathTextBox.Rebind("Text", this.profile.DataAccessInterface, "OutputPath", false, DataSourceUpdateMode.OnPropertyChanged);

            this.ormNamespaceTextBox.Rebind("Text", this.profile.ORM, "Namespace", false, DataSourceUpdateMode.OnPropertyChanged);
            this.ormOutputPathTextBox.Rebind("Text", this.profile.ORM, "OutputPath", false, DataSourceUpdateMode.OnPropertyChanged);
            this.ormProjectFileTextBox.Rebind("Text", this.profile.ORM, "VSProjectFile", false, DataSourceUpdateMode.OnPropertyChanged);
            this.ormObjectsFolderTextBox.Rebind("Text", this.profile.ORM, "ObjectsFolder", false, DataSourceUpdateMode.OnPropertyChanged);
            this.ormRepositoriesFolderTextBox.Rebind("Text", this.profile.ORM, "RepositoriesFolder", false, DataSourceUpdateMode.OnPropertyChanged);
            this.ormEnabledCheckBox.Rebind("Checked", this.profile.ORM, "Enabled", false, DataSourceUpdateMode.OnPropertyChanged);

            this.EndUpdate(true);
        }



        private void ProfileControl_Load(object sender, EventArgs e)
        {
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            using (AddTableExcludeForm f = new AddTableExcludeForm())
            {
                if (f.ShowDialog(this.Parent) == DialogResult.OK)
                {
                    this.excludesBindingList.Add(f.Exclude);
                }
            }
        }

        private void deleteButton_Click(object sender, EventArgs e)
        {
            this.excludesBindingList.Remove(excudesListBox.Text);
        }

        private void enumerationsListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.deleteButton.Enabled = (this.excudesListBox.SelectedIndex >=0);
        }

        private void pluralizerComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.profile.Pluralizer = (PluralType)this.pluralizerComboBox.SelectedIndex;
        }
    }
}
