﻿//-----------------------------------------------------------------------
// <copyright file="DBTableTraceExtensions.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator.DataAccess
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;

    /// <summary>
    /// Class for generating trace statements.
    /// </summary>
    public static class DBTableTraceExtensions
    {
        public static GenerationProfile Profile { get; set; }

        public static string DATraceGet(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            int i = 1;
            StringBuilder stmt = new StringBuilder();
            StringBuilder args = new StringBuilder();
            stmt.Append(@"Trace.TraceInformation(""{0}.Get(");
            foreach (var column in table.PrimaryKey.Values)
            {
                stmt.Append(column.Name + @"=");
                stmt.Append("{" + i++ + "}");

                args.AppendCommaList(column.Name.ToCamel());
            }

            stmt.Append(@")"",this.GetType().Name, " + args.ToString() + ");" + Environment.NewLine);
            return stmt.ToString();
        }

        public static string DATraceGetAll(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            return @"Trace.TraceInformation(""{0}.GetAll()"", this.GetType().Name);" + Environment.NewLine;
        }

        public static string DATraceInsert(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            return @"Trace.TraceInformation(""{0}.Insert()"", this.GetType().Name);" + Environment.NewLine;
        }

        public static string DATraceDelete(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            return @"Trace.TraceInformation(""{0}.Delete()"", this.GetType().Name);" + Environment.NewLine;
        }

        public static string DATraceSave(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            return @"Trace.TraceInformation(""{0}.Save()"", this.GetType().Name);" + Environment.NewLine;
        }

        public static string DATraceCreate(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            return @"Trace.TraceInformation(""{0}.Create()"", this.GetType().Name);" + Environment.NewLine;
        }

        public static string DATraceUpdate(this DBTable table)
        {
            if (!Profile.Trace)
            {
                return "";
            }

            return @"Trace.TraceInformation(""{0}.Update()"", this.GetType().Name);" + Environment.NewLine;
        }
    }
}
