﻿//-----------------------------------------------------------------------
// <copyright file="DataAccessGenerator.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator.DataAccess
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Diagnostics;
    using System.IO;
    using Dax.Generator.DataAccess.Templates;
    using Dax.Database;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DataAccessGenerator : Generator
    {

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DataAccessGenerator class.
        /// </summary>
        public DataAccessGenerator(GenerationProfile profile, DBDatabase database)
            : base(profile, database, profile.DataAccess.VSProjectFile, "Data Access", true)
        {
            // Make sure the extensions are setup.
            DBTableTraceExtensions.Profile = profile;
            DBTableExtensions.Profile = profile;
        }
        #endregion

        protected override void GenerationStart()
        {
            // Clear output directories of files.
            if (this.Profile.DataAccess.ClearOutputDirectories)
            {
                Trace.WriteLine("Clearing output directories...");
                DirectoryHelper.ClearDirectory(this.Profile.DataAccess.RecordsFolderFullPath);
                DirectoryHelper.ClearDirectory(this.Profile.DataAccess.RepositoriesFolderFullPath);
                DirectoryHelper.ClearDirectory(this.Profile.DataAccess.BuildersFolderFullPath);
            }

            // Make sure output directories exist.
            Directory.CreateDirectory(this.Profile.DataAccess.RecordsFolderFullPath);
            Directory.CreateDirectory(this.Profile.DataAccess.RepositoriesFolderFullPath);
            Directory.CreateDirectory(this.Profile.DataAccess.BuildersFolderFullPath);

            this.Session.Add("namespace", this.Profile.DataAccess.Namespace);
        }

        protected override void GenerateDatabase(DBDatabase database)
        {
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.OutputFilePath("DataAccess.cs"), typeof(DataAccess), this.Database));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.RecordFilePath("DataRecord.cs"), typeof(DataRecordClass), this.Database));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.RepositoryFilePath("DataAccessRepository.cs"), typeof(DataAccessRepositoryClass), this.Database));
        }

        protected override void GenerateTable(DBTable table)
        {
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.RecordFilePath(table.DARecordClassName() + ".cs"), typeof(DataRecord), Database, table));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.BuilderFilePath(table.DABuilderClassName() + ".cs"), typeof(DataRecordBuilder), Database, table));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.RepositoryFilePath(table.DARepositoryClassName() + ".cs"), typeof(DataAccessRepository), Database, table));
        }
    }
}
