﻿//-----------------------------------------------------------------------
// <copyright file="DataAccessTestGenerator.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator.DataAccessTest
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Text;
    using Dax.Database;
    using Dax.Generator.DataAccess.Templates;
    using Dax.Generator.DataAccessTest.Templates;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DataAccessTestGenerator : Generator
    {
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DataAccessTestGenerator class.
        /// </summary>
        public DataAccessTestGenerator(GenerationProfile profile, DBDatabase database)
            : base(profile, database, profile.DataAccess.TestVSProjectFile, "Data Access Test", true)
        {
        }
        #endregion

        protected override void GenerationStart()
        {
            // Make sure output directories exist.
            if (!string.IsNullOrEmpty(this.Profile.DataAccess.TestRecordsFolderFullPath)) Directory.CreateDirectory(this.Profile.DataAccess.TestRecordsFolderFullPath);
            if (!string.IsNullOrEmpty(this.Profile.DataAccess.TestRepositoriesFolderFullPath)) Directory.CreateDirectory(this.Profile.DataAccess.TestRepositoriesFolderFullPath);
            if (!string.IsNullOrEmpty(this.Profile.DataAccess.TestBuildersFolderFullPath)) Directory.CreateDirectory(this.Profile.DataAccess.TestBuildersFolderFullPath);

            this.Session.Add("namespace", this.Profile.DataAccess.TestNamespace);
        }

        protected override void GenerateDatabase(DBDatabase database)
        {
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.TestOutputFilePath("DataAccess.cs"), typeof(DataAccess), this.Database));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.TestRecordFilePath("DataRecord.cs"), typeof(DataRecordClass), this.Database));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.TestRepositoryFilePath("DataAccessRepository.cs"), typeof(DataAccessRepositoryClass), this.Database));
        }

        protected override void GenerateTable(DBTable table)
        {
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.TestRecordFilePath(table.DARecordClassName() + ".cs"), typeof(DataRecord), Database, table));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.TestBuilderFilePath(table.DABuilderClassName() + ".cs"), typeof(DataRecordBuilder), Database, table));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.DataAccess.TestRepositoryFilePath(table.DARepositoryClassName() + ".cs"), typeof(DataAccessTestRepository), Database, table));
        }
    }
}
