﻿//-----------------------------------------------------------------------
// <copyright file="DBRelationshipExtensions.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public static class DBRelationshipExtensions
    {
        public static GenerationProfile Profile { get; set; }

        /// <summary>
        /// Returns whether there are multiple relationships with 
        /// the same parent/referenced tables.
        /// </summary>
        /// <param name="relationship"></param>
        /// <returns></returns>
        public static bool HasMultipleReferences(this DBRelationship relationship)
        {
            return (from t in relationship.ParentTable.ParentRelationships
                    where t.ReferencedTable == relationship.ReferencedTable
                    select t).Count() > 1;
        }

        public static string ORMParentObjectPrivateVariableName(this DBRelationship relationship)
        {
            if (relationship.HasMultipleReferences())
            {
                return "m" + relationship.Name;
            }
            else
            {
                return relationship.ReferencedTable.ORMObjectPrivateVariableName();
            }
        }

        public static string ORMParentPropertyName(this DBRelationship relationship)
        {
            if (relationship.ParentTable == relationship.ReferencedTable)
            {
                return relationship.Name;
            }
            else
            {
                if (relationship.HasMultipleReferences())
                {
                    return relationship.Name;
                }
                else
                {
                    return relationship.ReferencedTable.Name.Singlular();
                }
            }
        }

        public static string ORMChildrenPropertyName(this DBRelationship relationship)
        {
            if (relationship.HasMultipleReferences())
            {
                return relationship.Name.Plural();
            }
            else
            {
                return relationship.ParentTable.ORMRepositoryContextPropertyName();
            }
        }

        public static string NullKeyValuesCheckExpression(this DBRelationship relationship, string qualifier)
        {
            StringBuilder s = new StringBuilder();

            foreach (var column in relationship.Columns.Values)
            {
                if (column.ParentColumn.IsNullable)
                {
                    if (column.ParentColumn.IsString())
                    {
                        s.AppendFormat(" && !string.IsNullOrEmpty({0}.{1})", qualifier, column.ParentColumn.Name);

                    }
                    else
                    {
                        s.AppendFormat(" && {0}.{1}.HasValue", qualifier, column.ParentColumn.Name);
                    }
                }
            }

            return s.ToString();
        }

        public static string DARepositoryGetByName(this DBRelationship relationship)
        {
            if (relationship.HasMultipleReferences())
            {
                return relationship.Name;
            }
            else
            {
                return relationship.ReferencedTable.DAReferencedName();
            }
        }

        public static string GetStoredProcedureName(this DBRelationship relationship)
        {
            return "[" + relationship.ParentTable.Schema.Name + "].[" + Profile.StoredProcedures.Prefix + relationship.ParentTable.Name + ".GetBy" + relationship.DARepositoryGetByName() + "]";
        }

        public static string GetByParameterList(this DBRelationship relationship)
        {
            StringBuilder s = new StringBuilder();

            foreach (var column in relationship.ParentColumns)
            {
                s.AppendCommaList(string.Format("{0} {1}", SqlTypeConverter.FromSql(column.SystemType.Name, column.IsNullable), column.Name.ToCamel()));
            }

            return s.ToString();
        }

        public static string KeyList(this DBRelationship relationship, string qualifier)
        {
            StringBuilder s = new StringBuilder();

            foreach (var column in relationship.ParentColumns)
            {
                if (column.IsNullable && !column.IsString())
                {
                    s.AppendCommaList(string.Format("{0}.{1}.Value", qualifier, column.Name));
                }
                else
                {
                    s.AppendCommaList(string.Format("{0}.{1}", qualifier, column.Name));
                }
            }
            return s.ToString();
        }
    }
}
