﻿//-----------------------------------------------------------------------
// <copyright file="DatabaseValidator.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;
    using System.Diagnostics;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DatabaseValidator
    {
        private GenerationProfile profile;
        private DBDatabase database;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DatabaseValidator class.
        /// </summary>
        public DatabaseValidator(DBDatabase database, GenerationProfile profile)
        {
            this.database = database;
            this.profile = profile;
        }
        #endregion

        public bool Validate()
        {
            Trace.TraceInformation("------ Database validation started ------");
            
            bool result = true;

            this.TableExclusions();
            this.Columns();

            Trace.TraceInformation("------ Database validation complete ------");
            return result;
        }

        private void TableExclusions()
        {
            foreach (var name in this.profile.TableExclusions)
            {
                this.database.RemoveTable(name);
            }
        }

        private void Columns()
        {
            Trace.TraceInformation("Validating columns...");
            foreach (DBTable table in database.Tables.Values)
            {
                List<DBColumn> deletes = new List<DBColumn>();
                foreach (var item in table.Columns.Values)
                {
                    if (!SqlTypeConverter.CanConvert(item.SystemType.Name))
                    {
                        Trace.TraceInformation("Cannot convert type {0}, ignoriong colunm {1} in table {2}.", item.SystemType.Name, item.Name, table.Name);
                        deletes.Add(item);
                    }

                }
                foreach (var item in deletes)
                {
                    table.RemoveColumn(item);
                }
            }
        }
        private void Relationships()
        {
            Trace.TraceInformation("Validating relationships...");

            List<DBRelationship> deletes = new List<DBRelationship>();
            foreach (DBRelationship relationship in database.Relationships.Values)
            {
                if (relationship.Columns.Count > 1)
                {
                    Trace.TraceInformation("Relationship {0} contains more than one parent column.", relationship.Name);
                    deletes.Add(relationship);
                }
            }

            foreach (DBRelationship relationship in deletes)
            {
                Trace.TraceInformation("Removing relationship {0}", relationship.Name);
                database.RemoveRelationship(relationship);
            }
        }
    }
}
