﻿//-----------------------------------------------------------------------
// <copyright file="Generator.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Data.SqlClient;
    using Dax.Generator.DataAccess;
    using System.Diagnostics;
    using System.IO;
    using System.Collections.Concurrent;

    /// <summary>
    /// Class to process a list of GenerationFiles, execute the template and write the output file.
    /// </summary>
    public class FileGenerator
    {
        private GenerationProfile profile;
        private IDictionary<string, object> session;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the Generator class.
        /// </summary>
        public FileGenerator(GenerationProfile profile, IDictionary<string, object> session)
        {
            this.profile = profile;
            this.session = session;
        }
        #endregion

        internal void Generate(ConcurrentBag<GenerationFile> generationFiles)
        {
            int fileCount = 0;
            foreach (var item in generationFiles)
            {
                GenerateFile(item);
                fileCount++;
            }
        }

        private void GenerateFile(GenerationFile generationFile)
        {
            if (generationFile.KeepExisting && File.Exists(generationFile.FileName))
            {
                return;
            }

            Trace.TraceInformation("Generating " + generationFile.FileName);

            ITemplate template = Activator.CreateInstance(generationFile.Template) as ITemplate;
            template.Session = this.session;
            template.Session["table"] = generationFile.Table;

            UpdateFile(generationFile.FileName, template.TransformText());
        }

        private void UpdateFile(string fileName, string newText)
        {
            string existingText = string.Empty;

            if (File.Exists(fileName))
            {
                existingText = File.ReadAllText(fileName);
            }

            if (string.Compare(existingText, newText) != 0)
            {
                File.WriteAllText(fileName, newText);
            }
        }
    }
}
