﻿//-----------------------------------------------------------------------
// <copyright file="ORMGenerator.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator.ORM
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Diagnostics;
    using System.IO;
    using Dax.Generator.ORM.Templates;
    using Dax.Database;

    /// <summary>
    /// ORM generator class.
    /// </summary>
    public class ORMGenerator : Generator
    {
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the ORMGenerator class.
        /// </summary>
        public ORMGenerator(GenerationProfile profile, DBDatabase database)
            : base(profile, database, profile.ORM.VSProjectFile, "ORM", true)
        {
        }
        #endregion

        protected override void GenerationStart()
        {
            // Make sure output directories exist.
            Directory.CreateDirectory(this.Profile.ORM.OutputPath);
            Directory.CreateDirectory(this.Profile.ORM.RepositoriesFolderFullPath);
            Directory.CreateDirectory(this.Profile.ORM.ObjectsFolderFullPath);

            this.Session.Add("namespace", this.Profile.ORM.Namespace);
        }

        protected override void GenerateDatabase(DBDatabase database)
        {
            this.GenerationFiles.Add(new GenerationFile(this.Profile.ORM.OutputFilePath(this.Database.Name + "ORMContext.cs"), typeof(ORMContext), this.Database));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.ORM.ObjectsFilePath("ORMObject.cs"), typeof(ORMObjectClass), this.Database));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.ORM.RepositoriesFilePath("ORMRepository.cs"), typeof(ORMRepositoryClass), this.Database));
        }

        protected override void GenerateTable(DBTable table)
        {
            this.GenerationFiles.Add(new GenerationFile(this.Profile.ORM.RepositoriesFilePath(table.ORMRepositoryClassName() + ".cs"), typeof(ORMRepository), this.Database, table));
            this.GenerationFiles.Add(new GenerationFile(this.Profile.ORM.ObjectsFilePath(table.ORMObjectClassName() + ".cs"), typeof(ORMObject), this.Database, table));
        }

        private void InialiseOutput()
        {
            // Clear output directories of files.
            if (this.Profile.DataAccess.ClearOutputDirectories)
            {
                Trace.WriteLine("Clearing output directories...");
                DirectoryHelper.ClearDirectory(this.Profile.ORM.ObjectsFolderFullPath);
                DirectoryHelper.ClearDirectory(this.Profile.ORM.RepositoriesFolderFullPath);
            }

            // Make sure output directories exist.
            Directory.CreateDirectory(this.Profile.ORM.ObjectsFolderFullPath);
            Directory.CreateDirectory(this.Profile.ORM.RepositoriesFolderFullPath);
        }
    }
}
