﻿//-----------------------------------------------------------------------
// <copyright file="Program.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Reflection;
    using System.Text;
    using System.Xml.Serialization;
    using Dax;
    using Dax.Database;
    using Dax.Generator.DataAccess;
    using Dax.Generator.DataAccessTest;
    using Dax.Generator.ORM;

    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length != 1)
            {
                ShowHelp();
                return;
            }

            Run(args[0]);
        }

        static void ShowHelp()
        {
            Console.WriteLine("");
            Console.WriteLine("Dax Generator Version " + Assembly.GetExecutingAssembly().GetName().Version.ToString());
            Console.WriteLine("");
            Console.WriteLine("Use:");
            Console.WriteLine("daxp [daxprofile]");
        }

        static void Run(string fileName)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(GenerationProfile));
            GenerationProfile profile = serializer.Deserialize(new StreamReader(fileName)) as GenerationProfile;

            SqlServerDBBuilder builder = new SqlServerDBBuilder(profile.ConnectionString);
            DBDatabase database = builder.Get();

            DatabaseValidator validator = new DatabaseValidator(database, profile);
            if (!validator.Validate())
            {
                return;
            }

            DBTableExtensions.Profile = profile;
            DBDatabaseExtensions.Profile = profile;
            DBColumnExtensions.Profile = profile;
            DBRelationshipExtensions.Profile = profile;
            DBProcedureExtensions.Profile = profile;
            Pluralizer.Mode = profile.Pluralizer;

            List<Generator> generators = new List<Generator>();
            generators.Add(new StoredProcedureGenerator(profile, database));
            generators.Add(new DataAccessGenerator(profile, database));
            generators.Add(new DataAccessTestGenerator(profile, database));
            generators.Add(new DataAccessInterfaceGenerator(profile, database));

            if (profile.ORM.Enabled)
            {
                generators.Add(new ORMGenerator(profile, database));
            }

            foreach (var generator in generators)
            {
                generator.Generate();
            }
        }
    }
}
