﻿//-----------------------------------------------------------------------
// <copyright file="SqlTypeConverter.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public sealed class SqlTypeConverter
    {
        private static Dictionary<string, string> fromSql = new Dictionary<string, string>()
        {
            {"nvarchar", "string"},
            {"varchar", "string"},
            {"nchar", "string"},
            {"char", "string"},
            {"text", "string"},
            {"bit", "bool"},
            {"int", "int"},
            {"datetime", "DateTime"},
            {"timestamp", "byte[]"},
            {"decimal", "decimal"},
            {"money", "decimal"},
            {"uniqueidentifier", "Guid"},
            {"tinyint", "byte"},
            {"smallint", "Int16"},
            {"smallmoney", "Decimal"}
        };

        private static Dictionary<string, string> fromSqlForGet = new Dictionary<string, string>()
        {
            {"nvarchar", "GetString"},
            {"varchar", "GetString"},
            {"nchar", "GetString"},
            {"char", "GetString"},
            {"text", "GetString"},
            {"bit", "GetBoolean"},
            {"int", "GetInt32"},
            {"datetime", "GetDateTime"},
            {"timestamp", "GetValue"},
            {"decimal", "GetDecimal"},
            {"money", "GetDecimal"},
            {"uniqueidentifier", "GetGuid"},
            {"smallint", "GetInt16"},
            {"tinyint", "GetByte"},
            {"smallmoney", "GetDecimal"}
        };

        private static Dictionary<string, string> sqlDbType = new Dictionary<string, string>()
        {
            {"nvarchar", "DbType.String"},
            {"varchar", "DbType.String"},
            {"nchar", "DbType.String"},
            {"char", "DbType.String"},
            {"text", "DbType.String"},
            {"bit", "DbType.Boolean"},
            {"int", "DbType.Int32"},
            {"datetime", "DbType.DateTime"},
            {"timestamp", "DbType.Object"},
            {"decimal", "DbType.Decimal"},
            {"money", "DbType.Decimal"},
            {"uniqueidentifier", "DbType.Guid"},
            {"smallint", "DbType.Int16"},
            {"tinyint", "DbType.Byte"},
            {"smallmoney", "DbType.Decimal"}
        };

        public static bool CanConvert(string type)
        {
            return fromSql.ContainsKey(type);
        }

        public static string FromSql(string type, bool nullable)
        {
            string s = fromSql[type];
            if (nullable && s != "string")
            {
                s += '?';
            }
            return s;
        }

        public static string FromSqlForGet(string type)
        {
            return fromSqlForGet[type];
        }

        public static string SqlDbType(string type)
        {
            return sqlDbType[type];
        }
    }
}
