﻿//-----------------------------------------------------------------------
// <copyright file="StoredProcedureGenerator.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;
    using System.Diagnostics;
    using System.IO;
    using Dax.Generator.StoredProcedures.Templates;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class StoredProcedureGenerator : Generator
    {
        private MemoryStream ms;
        private StreamWriter writer;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the StoredProcedureGenerator class.
        /// </summary>
        public StoredProcedureGenerator(GenerationProfile profile, DBDatabase database)
            :base(profile,database,"Stored Procedure", false)
        {
        }
        #endregion

        protected override void GenerationStart()
        {
            ms = new MemoryStream(20000);
            writer = new StreamWriter(ms);
        }

        protected override void GenerateDatabase(DBDatabase database)
        {
            Header h = new Header();
            writer.Write(h.TransformText());

            Dictionary<string, object> session = new Dictionary<string, object>();
            session.Add("prefix", this.Profile.StoredProcedures.Prefix);

            DeleteExisting de = new DeleteExisting();
            de.Session = session;
            writer.WriteLine();
            writer.Write(de.TransformText());
            
        }

        protected override void GenerateTable(DBTable table)
        {
            Trace.TraceInformation("Generating procedures for " + table.Name);

            writer.WriteLine();
            writer.WriteLine("--------------------------------------------------------------------------------");
            writer.WriteLine("-- " + table.Name);
            writer.WriteLine("--------------------------------------------------------------------------------");

            Delete d = new Delete();
            d.Profile = this.Profile;
            d.Table = table;
            writer.Write(d.TransformText());
            writer.WriteLine();

            if (table.UpdatableColumns().Count() > 0)
            {
                Update u = new Update();
                u.Profile = this.Profile;
                u.Table = table;
                writer.Write(u.TransformText());
                writer.WriteLine();

                Insert i = new Insert();
                i.Profile = this.Profile;
                i.Table = table;
                writer.Write(i.TransformText());
                writer.WriteLine();
            }

            Get g = new Get();
            g.Profile = this.Profile;
            g.Table = table;
            writer.Write(g.TransformText());
        }

        protected override void GenerationEnd()
        {
            writer.Flush();

            StreamReader reader = new StreamReader(ms);
            ms.Position = 0;
            this.UpdateFileIfChanged(this.Profile.StoredProcedures.OutputFile, reader.ReadToEnd());            
        }

        private void UpdateFileIfChanged(string fileName, string newText)
        {
            string existingText = string.Empty;

            if (File.Exists(fileName))
            {
                existingText = File.ReadAllText(fileName);
            }

            if (string.Compare(existingText, newText) != 0)
            {
                File.WriteAllText(fileName, newText);
            }
        }
    }
}
