﻿//----------------------------------------------------------------------------
// <copyright file="DatabaseSchema.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax.Schema
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DatabaseSchema : DatabaseObject
    {
        private Dictionary<string, DatabaseProcedure> proceduresByName = new Dictionary<string, DatabaseProcedure>();
        private Dictionary<int, DatabaseProcedure> proceduresByObjectId = new Dictionary<int, DatabaseProcedure>();
        private Dictionary<string, DatabaseTable> tablesByName = new Dictionary<string, DatabaseTable>();
        private Dictionary<int, DatabaseTable> tablesByObjectId = new Dictionary<int, DatabaseTable>();
        private Dictionary<int, DatabaseType> types = new Dictionary<int, DatabaseType>();
        private Dictionary<int, DatabaseRelationship> relationships = new Dictionary<int, DatabaseRelationship>();

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DatabaseSchema class.
        /// </summary>
        public DatabaseSchema(string name)
            : base(name)
        {
        }
        #endregion

        public void AddTable(DatabaseTable table)
        {
            this.tablesByName.Add(table.Name, table);
            this.tablesByObjectId.Add(table.ObjectId, table);
        }

        public void AddProcedure(DatabaseProcedure procedure)
        {
            this.proceduresByName.Add(procedure.Name, procedure);
            this.proceduresByObjectId.Add(procedure.ObjectId, procedure);
        }

        public void AddType(DatabaseType type)
        {
            this.types.Add(type.UserTypeId, type);
        }

        public void AddRelationship(DatabaseRelationship relationship)
        {
            if (!this.relationships.ContainsKey(relationship.ObjectId))
            {
                this.relationships.Add(relationship.ObjectId, relationship);
            }
        }

        public void RemoveRelationship(DatabaseRelationship relationship)
        {
            if (this.relationships.ContainsKey(relationship.ObjectId))
            {
                relationship.ParentTable.RemoveRelationship(relationship);
                relationship.ReferencedTable.RemoveRelationship(relationship);
                this.relationships.Remove(relationship.ObjectId);
            }
        }

        public void RemoveTable(string tableName)
        {
            if (this.tablesByName.ContainsKey(tableName))
            {
                this.RemoveTable(this.GetTableByName(tableName));
            }
        }

        public void RemoveTable(DatabaseTable table)
        {
            if (this.tablesByObjectId.ContainsKey(table.ObjectId))
            {
                // Remove any relationships to this table.
                List<int> deletes = new List<int>();
                foreach (var item in this.relationships.Values.Where(r => r.ParentTable == table || r.ReferencedTable == table))
                {
                    deletes.Add(item.ObjectId);
                }
                foreach (int objectId in deletes)
                {
                    this.relationships.Remove(objectId);
                }

                this.tablesByObjectId.Remove(table.ObjectId);
                this.tablesByName.Remove(table.Name);
            }
        }

        public DatabaseTable GetTableByObjectId(int objectId)
        {
            return tablesByObjectId[objectId];
        }

        public bool ContainsTable(int objectId)
        {
            return tablesByObjectId.ContainsKey(objectId);
        }

        public DatabaseTable GetTableByName(string name)
        {
            return tablesByName[name];
        }

        public DatabaseType GetType(int userTypeId)
        {
            return types[userTypeId];
        }


        public IEnumerable<DatabaseProcedure> Procedures
        {
            get
            {
                return this.proceduresByName.Values;
            }
        }

        public IEnumerable<DatabaseTable> Tables
        {
            get
            {
                return this.tablesByName.Values;
            }
        }

        public IEnumerable<DatabaseRelationship> Relationships
        {
            get
            {
                return this.relationships.Values;
            }
        }
    }
}
