﻿//----------------------------------------------------------------------------
// <copyright file="DatabaseTable.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax.Schema
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class DatabaseTable : DatabaseObject
    {
        private Dictionary<string, DatabaseColumn> columnsByName = new Dictionary<string, DatabaseColumn>();
        private Dictionary<int, DatabaseColumn> columnsById = new Dictionary<int, DatabaseColumn>();
        private Dictionary<int, DatabaseColumn> primaryKey = new Dictionary<int, DatabaseColumn>();
        private Dictionary<int, DatabaseRelationship> relationships = new Dictionary<int, DatabaseRelationship>();
        private int objectId;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DatabaseTable class.
        /// </summary>
        public DatabaseTable(string name, int objectId) 
            : base(name)
        {
            this.objectId = objectId;
        }
        #endregion

        public void AddPrimaryKeyColumn(int key_id, int column_id)
        {
            DatabaseColumn c = GetColumnById(column_id);
            c.IsPrimaryKey = true;
            primaryKey.Add(key_id, c);
        }

        public void AddColumn(DatabaseColumn column)
        {
            this.columnsByName.Add(column.Name, column);
            this.columnsById.Add(column.Id, column);
        }

        public void RemoveColumn(DatabaseColumn column)
        {
            this.columnsByName.Remove(column.Name);
            this.columnsById.Remove(column.Id);
        }

        public void AddRelationship(DatabaseRelationship relationship)
        {
            this.relationships.Add(relationship.ObjectId, relationship);
        }

        public void RemoveRelationship(DatabaseRelationship relationship)
        {
            this.relationships.Remove(relationship.ObjectId);
        }

        public DatabaseColumn GetColumnById(int id)
        {
            return columnsById[id];
        }

        public DatabaseColumn GetColumnByName(string name)
        {
            return columnsByName[name];
        }

        public DatabaseRelationship GetRelationshipById(int id)
        {
            return this.relationships[id];
        }

        public bool ContainsColumn(string name)
        {
            return this.columnsByName.ContainsKey(name);
        }

        public IEnumerable<DatabaseColumn> Columns
        {
            get
            {
                return this.columnsById.Values.OrderBy(x => x.Id).ToList();
            }
        }

        public IEnumerable<DatabaseRelationship> ParentRelationships
        {
            get
            {
                return this.relationships.Values.Where(x => x.ParentTable == this).ToList();
            }
        }

        public IEnumerable<DatabaseRelationship> ReferencedRelationships
        {
            get
            {
                return this.relationships.Values.Where(x => x.ReferencedTable == this).ToList();
            }
        }

        public List<DatabaseColumn> PrimaryKey
        {
            get
            {
                return this.primaryKey.Values.ToList();
            }
        }

        public bool HasPrimaryKey
        {
            get
            {
                return this.primaryKey.Count != 0;
            }
        }

        public bool CompositePrimaryKey
        {
            get
            {
                return this.primaryKey.Count > 1;
            }
        }

        public bool HasIdentity
        {
            get
            {
                return (this.Identity != null);
            }
        }

        public DatabaseColumn Identity
        {
            get
            {
                return (from c in this.Columns
                       where c.IsIdentity
                       select c).FirstOrDefault();
            }
        }

        /// <summary>
        /// Gets the ObjectId
        /// </summary>
        /// <value>ObjectId</value>
        public int ObjectId
        {
            get
            {
                return this.objectId;
            }
        }
    }
}
