﻿//-----------------------------------------------------------------------
// <copyright file="SchemaHelper.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Test
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;

    public class DatabaseHelper
    {
        public static DBDatabase BuildTestDatabase()
        {
            DBDatabase database = new DBDatabase("Test");

            // Set up schemas
            DBSchema schema =  new DBSchema(database, 0, "dbo");
            database.Schemas.Add(schema.Id, schema);

            // Set up the types.
            DBType dbInt = new DBType(database, 0, "int");
            DBType dbNVarChar = new DBType(database, 1, "nvarchar");
            DBType dbDateTime = new DBType(database, 2, "datetime");

            database.Types.Add(dbInt.Id, dbInt);
            database.Types.Add(dbNVarChar.Id, dbNVarChar);
            database.Types.Add(dbDateTime.Id, dbDateTime);

            DBTable customersTable = new DBTable(database, schema, 0, "Customers");
            customersTable.Columns.Add(1, new DBColumn(database, 0, "CustomerId") { SystemType = dbInt, IsIdentity = true , IsPrimaryKey = true});
            customersTable.Columns.Add(2, new DBColumn(database, 1, "CustomerName") { SystemType = dbNVarChar });
            customersTable.Columns.Add(3, new DBColumn(database, 2, "Updated") { SystemType = dbDateTime });
            customersTable.PrimaryKey.Add(1, customersTable.Columns[1]);
            database.Tables.Add(customersTable.Id, customersTable);


            DBTable ordersTable = new DBTable(database, schema, 1, "Orders");
            ordersTable.Columns.Add(1, new DBColumn(database, 0, "OrderId") { SystemType = dbInt, IsIdentity = true, IsPrimaryKey = true });
            ordersTable.Columns.Add(2, new DBColumn(database, 2, "Updated") { SystemType = dbDateTime });
            ordersTable.PrimaryKey.Add(1, ordersTable.Columns[1]);
            database.Tables.Add(ordersTable.Id, ordersTable);

            DBTable orderLinesTable = new DBTable(database, schema, 2, "OrderLines");
            orderLinesTable.Columns.Add(1, new DBColumn(database, 0, "OrderId") { SystemType = dbInt, IsIdentity = true, IsPrimaryKey = true });
            orderLinesTable.Columns.Add(2, new DBColumn(database, 1, "OrderLineId") { SystemType = dbInt, IsPrimaryKey = true });
            orderLinesTable.Columns.Add(3, new DBColumn(database, 2, "OrderQty") { SystemType = dbInt });
            orderLinesTable.Columns.Add(4, new DBColumn(database, 3, "Updated") { SystemType = dbDateTime });
            orderLinesTable.PrimaryKey.Add(1, orderLinesTable.Columns[1]);
            orderLinesTable.PrimaryKey.Add(2, orderLinesTable.Columns[2]);
            database.Tables.Add(orderLinesTable.Id, orderLinesTable);

            DBTable productsTable = new DBTable(database, schema, 3, "Products");
            productsTable.Columns.Add(1, new DBColumn(database, 0, "ProductId") { SystemType = dbInt, IsPrimaryKey = true });
            productsTable.Columns.Add(2, new DBColumn(database, 1, "ProductName") { SystemType = dbNVarChar });
            productsTable.Columns.Add(3, new DBColumn(database, 2, "Updated") { SystemType = dbDateTime });
            productsTable.PrimaryKey.Add(1, productsTable.Columns[1]);
            database.Tables.Add(productsTable.Id, productsTable);


            return database;
        }
    }
}
